/******************************************************************************/
/*  bc635PCI TFP card Configuration  utility developed for IRIX 6.3  on       */
/*  Silicon Graphics Octane system. This utility uses Generic PCI driver      */
/*  provided (usrpci) with IRIX  Operating System                             */
/*  The utility uses two Shell variables or two command Line arguments for    */
/*  XIO Node ID and PCI Slot ID which are essential for normal functioning of */
/*  utility and device                                                        */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <sys/mman.h>
#include "bctime.h"
#define  XIONODE  "XIONODE"                             /* Shell Variable for XIO node Id */
#define  PCISLOT  "BCISLOT"								/* Shell Variable for PCI Slot Id */
#define  BC_VENDOR_ID        0x12e2                     /* PCI Vendor ID for bc635 */
#define  BC635_DEVICE_ID     0x4013                     /* PCI Device ID for bc635 */
/* #define TEST   1   */

int DatmConfigDevice, Base1Device, Base2Device;
int xnid, pcisid, venid, devid;

char *ConfigDevName ="/hw/node/xtalk/13/pci/2/usrpci/config";  /* usrpci device name for bc635 Interface */
char *Base1Devname ="/hw/node/xtalk/13/pci/2/usrpci/default";
char *Base2Devname ="/hw/node/xtalk/13/pci/2/usrpci/base/1";

/* Base Register Labels */
char *bc635devreglabel[17] = { "TIMEREQ ",      
                               "EVENTREQ",
                               "UNLOCK  ", 
                               "RESERVED",
                               "CONTROL ",
                               "ACK     ",
                               "MASK    ", 
                               "INTSTAT ", 
                               "MINSTRB ", 
                               "MAJSTRB ", 
                               "RESERVED", 
                               "RESERVED", 
                               "TIME0   ", 
                               "TIME1   ", 
                               "EVENT0  ", 
                               "EVENT1  ",
                               "        "};
/* bc635 Status field Labels */

char *bc635statuslabel[23] = { "Timing Mode :",
                               "Time Format :",
                               "########### :",
                               "Year        :",
                               "Per.Output  :",
                               "TCode Format:",
                               "TCode Modul.:",
                               "Toffset     :",
                               "UTC Info    :",
                               "########### :",
                               "########### :",
                               "########### :",
                               "########### :",
                               "########### :",
                               "########### :",
                               "TFP Version :",
                               "Clock       :",
                               "JamSync     :",
                               "########### :",
                               "Disciplining:",
                               "DAC         :",
                               "########### :",
                               "        "};
void  *DatumConfigAddr; /* Memory map pointer to bc635PCI device Configuration */
void  *Base1Addr;       /*Memory map pointer to bc635PCI device BAR */
void  *Base2Addr;       /* Memory map Pointer to bc635PCI device DPRAM */

char  *Config, *Base1, *Base2;
int   *bctime;
int   DatumDeviceRegisters[16];
int   cmdslct=99;
struct TFPCommand {
       unsigned char cID;
       union  {
             unsigned char UINT8;
             char          INT8;
             unsigned short UINT16;
             short         INT16;
             unsigned long UINT32;
             long          INT32;
             float         FLOAT32;
             double        DOUBLE64;
             } cmdarg ;
} bc635pci,bc635status;

unsigned char  cptr[17] ; 
 
/* Structures for Setting & getting time from bc635PCI board & IRIX System */

time_t curtime, dtmtime ;
struct timeval tp;
struct timezone tz;
/**************************************************************************/
int OpenDatumpPCIDevices()
{

if ((DatmConfigDevice=open (ConfigDevName, O_RDWR)) < 0) {
   return(0);
   }
if ((Base1Device=open (Base1Devname, O_RDWR)) <  0 ) {
   return(0);
   }
if ((Base2Device=open (Base2Devname, O_RDWR))  < 0) {
   return(0);
   }
return (1);
}
/**************************************************************************/
int ConfigMapDatumPCIDevice()
{
if((DatumConfigAddr  =
    mmap(NULL, (size_t)0x40, (int)(PROT_READ),
        (int) MAP_PRIVATE,DatmConfigDevice, (off_t)0x00)) != MAP_FAILED)
     {
     Config = (char *)((__uint64_t) DatumConfigAddr + ((pcisid&3)*0x1000));

#ifdef TEST
     printf ("\n Configuration Address for Datum :%x" ,Config );
#endif

     }

else {
     printf("\n Configuration Mapping Failed for BCDevice");
     return(0);
     }
return (1);
}
/**************************************************************************/
int MemoryMapDatumPCIDevices ()
{
if((Base1Addr=mmap(NULL, (size_t)0x40, (int)(PROT_READ|PROT_WRITE),
                  (int) MAP_SHARED, Base1Device, (off_t)0x00)) != MAP_FAILED) 
{

#ifdef TEST
   printf ("\n Base-1-address for Datum :%x" , Base1Addr);
#endif

   Base1=(char *)Base1Addr;
   bctime = (int *) Base1Addr;
}
else 
{
   printf("\n Device  Address Register  Mapping Failed for BCDevice");
   return(0);
}
if((Base2Addr= mmap(NULL, (size_t)0x800, (int)(PROT_READ| PROT_WRITE),
                   (int) MAP_SHARED,Base2Device, (off_t)0x00)) != MAP_FAILED) 
{
   Base2=(char *)Base2Addr;

#ifdef TEST
   printf ("\n Base-2-address for Datum :%x" , Base2Addr);
#endif

}
else 
{
   printf("\n Dual Port RAM Mapping Failed for BCDevice");
   return(0);
}
return (1);
}

void DumpDatumPCIDeviceConfiguration() 
{
printf("\n Card Configuration ");
printf("\n PCI Device ID. :0X%02x%02x", Config[0], Config[1]);
printf("\n PCI Vendor ID. :0X%02x%02x", Config[2], Config[3]);
printf("\n PCI Base Addr. :0X%02x%02x%02x%02x", Config[16], Config[17], Config[18], Config[19]);
}
/**************************************************************************/
void ControlRegDump()
{
int i; 
printf("\n Datum PCI-BC635  Control Registers Dump ");
printf("\n ------------------------------------------");
for (i=0;i<0x40;i+=4) 
     printf("\n %s ->%02x %02x %02x %02x",  bc635devreglabel[i/4], Base1[i],  
		                               Base1[i+1],  Base1[i+2],  Base1[i+3]);
     
curtime=time(NULL);
dtmtime=(*(bctime+13));
printf("\n [%x] -> %s", bctime[13],ctime(&dtmtime));
}
/**************************************************************************/ 
void DatumTFPDPRAMDump()
{
int i; 
printf("\n Datum PCI-BC635  Dual Port RAM  Dump ");
printf("\n -------------------------------------");
for (i=0;i<0x800;i+=4)  
{
  if (i % 0x10 == 0) printf ("\n [%03x] : ",i);
  printf("%02x %02x %02x %02x ",  Base2[i],  Base2[i+1],  Base2[i+2],  Base2[i+3]);
}

}
/**************************************************************************/

int closeAndExit()
{

munmap(DatumConfigAddr,0x40 );
munmap(Base1Addr,0x40);
munmap(Base2Addr,0x800);
close(DatmConfigDevice);
close(Base1Device);
close(Base2Device);
return (1);
}
/**************************************************************************/
void SendCommand(int nbytes)
{
 int i;
 if (nbytes > 4) return;
 memcpy (cptr, &bc635pci, sizeof(bc635pci));
 for (i=0;i<sizeof(bc635pci);i++) printf ("%02x ", cptr[i]);
 Base2[0x101] = cptr[0];
 Base2[0x100] = cptr[(sizeof(bc635pci)/2)]; 
 if(nbytes >= 2) Base2[0x107] = cptr[(sizeof(bc635pci)/2)+1];
 if(nbytes >= 4) {
   Base2[0x106] = cptr[(sizeof(bc635pci)/2)+2];
   Base2[0x105] = cptr[(sizeof(bc635pci)/2)+3];
   }
 memset(Base1+0x14,0x01, 4); 
 ControlRegDump();
 memset(Base1+0x14,0x80, 4); 
 DatumTFPDPRAMDump(); 
}
/**************************************************************************/
void GetTFPstatus (int datareq)
{

 switch (datareq) {
       case 0x12 :
       case 0x19 :
       case 0x1a :
       case 0x1b :
       case 0x1c :
       case 0x1d :
       case 0x1e :
       case 0x22 :/* printf("\n ############ # ############ ");*/ break;
       default :  
		   {
            bc635pci.cID = COM_GETDATA;
            bc635pci.cmdarg.UINT8 = bc635status.cID = datareq;
            SendCommand(1);
            bc635status.cID = Base2[0x81];
            bc635status.cmdarg.UINT8 = Base2[0x80];
            bc635status.cmdarg.UINT16 = (Base2[0x87] << 8)+Base2[0x80];
            bc635status.cmdarg.UINT32 = (Base2[0x85] << 24)+
			                            (Base2[0x86] << 16)+
			 						    (Base2[0x87] << 8)+Base2[0x80]; 
            printf("\n %s", bc635statuslabel[bc635status.cID - 0x10]);
            switch (bc635status.cID) 
			{
            case  COM_SETTIMINGMODE   :
				{
                switch (bc635status.cmdarg.UINT8) 
				{
                case MODE_TIMECODE : printf("IRIG - B"); break;
                case MODE_FREERUN  : printf("Free Running - 10 Mhz"); break;
                case MODE_EXT1PPS  : printf("1PPS External"); break;
                case MODE_GPS      : printf("GPS Mode"); break;
                default            : printf("Unknown") ;break;
				} break;
				}   
            case  COM_SETTIMEFORMAT   :
				{
                switch (bc635status.cmdarg.UINT8)
				{
                case TIME_DECIMAL : printf("Decimal"); break;
                case TIME_BINARY  : printf("Binary");break;
                default           : printf("Unknown") ;break;
				} break;
				}   
            case  COM_SETYEAR         :
				{
                printf("%u", bc635status.cmdarg.UINT16); break;
                }   
            case  COM_SETPERIODIC     :
				{
                switch (bc635status.cmdarg.UINT8)
				{
                case 0  : printf("Not Sync to 1PPS"); break;
                case 1  : printf("Sync to 1 PPS"); break;
                default : printf("Unknown") ;break;
				}  break;
				}   
            case  COM_SETTCFORMAT     :
				{
                switch (bc635status.cmdarg.UINT8)
				{
                case TC_IRIGA : printf("IRIG-A"); break;
                case TC_IRIGB : printf("IRIG-B"); break;
                case TC_2137  : printf("2137"); break;
                case TC_NASA36: printf("NASA36"); break;
                case TC_XR3   : printf("XR3"); break;
                default       : printf("Unknown") ;break;
				} 
                break;
				}   
            case  COM_SETTCMOD        :
				{
                switch (bc635status.cmdarg.UINT8)
				{
                case MOD_AM   : printf("Amplitude Modulation"); break;
                case MOD_DCLS : printf("Pulse Coded Modulation"); break;
                default       : printf("Unknown") ;break;
				} break;
                }   
            case  COM_SETTIMINGOFFSET :
				{
                 printf("%d", bc635status.cmdarg.UINT32); break;
				}   
            case  COM_UTCINFO         : break;
            case  COM_FIRMVER         : break;
            case  COM_SELCLOCK        :
				{
                switch (bc635status.cmdarg.UINT8)
				{
                case SELCLOCK_INT :  printf("Internal 10 Mhz VCX"); break;
                case SELCLOCK_EXT :  printf("External 10 Mhz"); break;
                default           :  printf("Unknown") ;break;
				}
                break;
                }   
            case  COM_JAMSYNCENABLE   :
				{
                switch (bc635status.cmdarg.UINT8)
				{
                case JAMSYNC_DIS : printf("Disabled"); break;
                case JAMSYNC_EN  : printf("Enabled "); break; 
				default          : printf("Unknown") ;break;
				}
                break;
				}   
            case  COM_DISCENABLE      :
				{
                 switch (bc635status.cmdarg.UINT8)
				 {
                 case DISCIPLINE_ENABLE : printf("Enabled"); break;
                 case DISCIPLINE_DISABLE: printf("Disabled"); break;
				 default                : printf("Unknown") ;break;
				 }
                 break;
				}   
            case  COM_SETDAC : printf("%4x", bc635status.cmdarg.UINT16); break;
            default          : break;
			} ;
        } break;
      } 
}
/**************************************************************************/
int DatumTFPCommadMenu()
{
printf("\n -----------------------------------------------");
printf("\n  BC635 - DPRAM Configuration Command Menu          ");
printf("\n ------------------------------------------------");
printf("\n Select TFP Timing Mode ..................( 1)");
printf("\n Select Timing Format ....................( 2)");
printf("\n Set Major Time  .........................( 3)");
printf("\n Set Year ................................( 4)");
printf("\n Set Periodic Output .....................( 5)");
printf("\n Select Time Code Format .................( 6)");
printf("\n Select Time Code Modulation type.........( 7)");
printf("\n Set Timing Offset  ......................( 8)");
printf("\n Request Data from TFP  ..................( 9)");
printf("\n Software Reset  .........................(10)");
printf("\n Select Clock Source......................(11)");
printf("\n Control Jam Sync  .......................(12)");
printf("\n Force Jam Sync  .........................(13)");
printf("\n Control Clock Disciplining  .............(14)");
printf("\n Load D/A Convertor ......................(15)");
printf("\n Set Disciplining Gain  ..................(16)");
printf("\n Dump Configuration Register..............(17)");
printf("\n Dump Control Resgisters .................(18)");
printf("\n Dump Dual Port RAM   ....................(19)");
printf("\n Get Time             ....................(20)");
printf("\n Set Time             ....................(21)");
printf("\n Show Time            ....................(22)");
printf("\n Exit ....................................(99)");
printf("\n Select Your Option ......................(");
scanf("%d", &cmdslct);
switch (cmdslct) {

case TFPMODE      : {
                    int modeselect=9;
                    printf("\n Timing Mode");
                    printf("\n--------------");
                    printf("\n Time Code - IRIG B ........ (1)");
                    printf("\n Freee Running - 10MHz (I/E) (2)");
                    printf("\n 1 PPS - External One PPS... (3)");
                    printf("\n GPS - GPS Antennal /Receiver(4)");
                    printf("\n Exit .......................(9)");
                    printf("\n Select Your Option .........(");
                    scanf("%d", &modeselect);
                    switch (modeselect) 
                    {
                    case 1 : 
#ifdef TEST  
                           printf("\n Timing Mode : Set to IRIG - B");
#endif
                            bc635pci.cmdarg.UINT8 = MODE_TIMECODE; break;
                    case 2 :
#ifdef TEST  
                           printf("\n Timing Mode : Set to Free Running - 10 Mhz");
#endif
                            bc635pci.cmdarg.UINT8 = MODE_FREERUN; break;
                    case 3 :
#ifdef TEST  
                           printf("\n Timing Mode : Set to 1PPS External");
#endif
                            bc635pci.cmdarg.UINT8 = MODE_EXT1PPS; break;
                    case 4 : 
#ifdef TEST  
                           printf("\n Timing Mode : Set To GPS Mode");
#endif
                            bc635pci.cmdarg.UINT8 = MODE_GPS; break;
                    case 9 : break; 
                    default : break;
                    }
                    bc635pci.cID = COM_SETTIMINGMODE;
                    if ((modeselect > 0) && (modeselect < 5)) SendCommand(1);
                    } break;
case TFPFORMAT    :{ 
                   int modeselect=9;
                    printf("\n Time Format ");
                    printf("\n--------------");
                    printf("\n Decimal ........ (1)");
                    printf("\n Binary  ........ (2)");
                    printf("\n Exit ............(9)");
                    printf("\n Select Option ...(");
                    scanf("%d", &modeselect);
                    switch (modeselect)
                    {
                    case 1 :
#ifdef TEST
                           printf("\n Time Format : Set to Decimal");
#endif
                            bc635pci.cmdarg.UINT8 = TIME_DECIMAL; break;
                    case 2 :
#ifdef TEST
                           printf("\n Time Format : Set to Binary");
#endif
                            bc635pci.cmdarg.UINT8 = TIME_BINARY;break;
                    case 9 : break;
                    default : break;
                    }
                    bc635pci.cID = COM_SETTIMEFORMAT;
                    if ((modeselect > 0) && (modeselect < 3)) SendCommand(1);
                    } break;

case SETMAJORTIME : { 
                    unsigned long bintim ; 
                    printf("\n Set Major Time  ");
                    printf("\n--------------");
                    printf("\n Enter Binary time to load in HEX formm (8-Digits Max):");
                    scanf("%08x", &bintim);
                    
#ifdef TEST  
                      printf("\n Set Major Time : Set to %08x", bintim);
#endif
                      
                    bc635pci.cID = COM_SETTIME;
                    bc635pci.cmdarg.UINT32 = bintim;
                    SendCommand(4);
                    }break;   
case SET_YEAR      : {
                    int year=1999;
                    printf("\n Set Year  ");
                    printf("\n--------------");
                    printf("\n Enter Year to load (1970-2050) :");
                    scanf("%04d", &year);
#ifdef TEST  
                      printf("\n Set Year  : Set to %04d", year);
#endif
                    bc635pci.cID =  0x12 ;
                    bc635pci.cmdarg.UINT16 = year;
                    if ((year >= 1970) && (year <= 2050)) SendCommand(2);
                    }break; 
case SETPRDOUT    : {
                    int pout;
                    printf("\n Set Periodic Output");
                    printf("\n---------------------");
                    printf("\n Do not Sync to 1 Sec PPS ...(1)");
                    printf("\n Sync to 1 Sec PPS ..........(2)");
                    printf("\n Exit .......................(9)");
                    printf("\n Select Your Option .........(");

                    scanf("%d", &pout);
                    switch (pout)
                    {
                    case 1 :
#ifdef TEST
                           printf("\n Set Periodic Output  :  Do Not Sync to 1PPS");
#endif
                            bc635pci.cmdarg.UINT8 = 0; break;
                    case 2 :
#ifdef TEST
                           printf("\n Set Periodic Output  :  Sync to 1PPS");
#endif
                            bc635pci.cmdarg.UINT8 = 1;break;
                    case 9 : break;
                    default : break;
                    }
       
                    bc635pci.cID = COM_SETPERIODIC;
                    if ((pout > 0) && (pout <3)) SendCommand(1);
                    }break; 
case SETTCFORMAT  : {
                    int tcformat;
                    printf("\n Set Time Code Format");
                    printf("\n---------------------");
                    printf("\n IRIG-A  ..............(1)");
                    printf("\n IRIG-B  ..............(2)");
                    printf("\n 2137    ..............(3)");
                    printf("\n NASA36  ..............(4)");
                    printf("\n XR3  .................(5)");
                    printf("\n Exit .................(9)");
                    printf("\n Select Your Option ...(");
                    scanf("%d", &tcformat);
                    switch (tcformat)
                    {
                    case 1 :
#ifdef TEST
                           printf("\n Set Time Code Format  : IRIG-A");
#endif
                            bc635pci.cmdarg.UINT8 = TC_IRIGA; break;
                    case 2 :
#ifdef TEST
                           printf("\n Set Time Code Format  : IRIG-B");
#endif
                            bc635pci.cmdarg.UINT8 = TC_IRIGB; break;
                    case 3 :
#ifdef TEST
                           printf("\n Set Time Code Format  : TC_2137");
#endif
                            bc635pci.cmdarg.UINT8 = TC_2137; break;
                    case 4 :
#ifdef TEST
                           printf("\n Set Time Code Format  : NASA36");
#endif
                            bc635pci.cmdarg.UINT8 = TC_NASA36; break;
                    case 5 :
#ifdef TEST
                           printf("\n Set Time Code Format  : XR3");
#endif
                            bc635pci.cmdarg.UINT8 = TC_XR3; break;



                    case 9 : break;
                    default : break;
                    }

                    bc635pci.cID = COM_SETTCFORMAT;
                    if ((tcformat > 0) && (tcformat <3))SendCommand(1);
                    } break; 
case SETTCMOD     : {
                    int tcmod;
                    printf("\n Set Time Code Modulation");
                    printf("\n---------------------");
                    printf("\n Amplitude Modulated...(1)");
                    printf("\n Pulse Code Moodulated.(2)");
                    printf("\n Exit .................(9)");
                    printf("\n Select Your Option ...(");

                    scanf("%d", &tcmod);
                    switch (tcmod)
                    {
                    case 1 :
#ifdef TEST
                           printf("\n Set Time Code Modulation   :  Amplitude");
#endif
                            bc635pci.cmdarg.UINT8 = MOD_AM; break;
                    case 2 :
#ifdef TEST
                           printf("\n Set Time Code Modulation   :  Pulse Coded");
#endif
                            bc635pci.cmdarg.UINT8 = MOD_DCLS ;break;
                    case 9 : break;
                    default : break;
                    }
                    bc635pci.cID = COM_SETTCMOD;
                    if ((tcmod > 0) && (tcmod <3))SendCommand(1);
                    } break; 
case SETTOFSET    : {
                    long  toffset;
                    printf("\n Set Timing Offset ");
                    printf("\n-------------------");
                    printf("\n Enter Time Offset in Microseconds(-9,999,9999 to +9,999,999): ");
                    printf("%ld", &toffset);
                    bc635pci.cID = COM_SETTIMINGOFFSET;
                    bc635pci.cmdarg.UINT32 = toffset ;
                    printf("\n Set Time offset  : %ld", toffset);
                    if ((toffset > -9999999l) && (toffset < 9999999l)) SendCommand(1);
                    }break; 
case REQDATA      : {
                    int i;
                    printf("\n bc635 TFP Status  ");
                    printf("\n-------------------");
                    for (i=0x10; i< 0x25 ; i++)  GetTFPstatus(i);
                    } break; 

case TFPRESET     : {
                    int i;
                    printf("\n Software Reset  ");
                    printf("\n-------------------");
                    bc635pci.cID = BC_SRESET ;
                    printf("\n Software RESET : Done");
                    SendCommand(1);
                    }break; 
case SELCLKSRC    : {
                    int clksrc;
                    printf("\n Select Clock Source");
                    printf("\n---------------------");
                    printf("\n Internal 10  Mhz VCXO.(1)");
                    printf("\n External 10 Mhz Clock.(2)");
                    printf("\n Exit .................(9)");
                    printf("\n Select Your Option ...(");
                    scanf("%d", &clksrc);

                    switch (clksrc)
                    {
                    case 1 :
#ifdef TEST
                           printf("\n Set Clock Source   :  Internal 10 Mhz VCX");
#endif
                            bc635pci.cmdarg.UINT8 = SELCLOCK_INT; break;
                    case 2 :
#ifdef TEST
                           printf("\n Set Clock Source   :  External 10 Mhz" );
#endif
                            bc635pci.cmdarg.UINT8 = SELCLOCK_EXT ;break;
                    case 9 : break;
                    default : break;
                    }
                    bc635pci.cID = COM_SELCLOCK;
                    if ((clksrc > 0) && (clksrc < 3)) SendCommand(1);
                    }break; 
case CTRLJS       : {
                    int ctrljsync;
                    printf("\n Control Jam Sync ");
                    printf("\n---------------------");
                    printf("\n Jam Sync  Disable  ...(1)");
                    printf("\n Jam Sync  Enable   ...(2)");
                    printf("\n Exit .................(9)");
                    printf("\n Select Your Option ...(");
                    scanf("%d", &ctrljsync);

                    switch (ctrljsync)
                    {
                    case 1 :
#ifdef TEST
                           printf("\n Control Jam Sync.   :  Jam Sync Disabled");
#endif
                            bc635pci.cmdarg.UINT8 = JAMSYNC_DIS; break;
                    case 2 :
#ifdef TEST
                           printf("\n Control Jam Sync.   :  Jam Sync Enabled");
#endif
                            bc635pci.cmdarg.UINT8 = JAMSYNC_EN ;break;
                    case 9 : break;
                    default : break;
                    }

                    bc635pci.cID = COM_JAMSYNCENABLE;
                    if ((ctrljsync > 0) && (ctrljsync < 3)) SendCommand(1);
                    }break; 
case FORCJS       : {
                    int i;
                    printf("\n Force Jam  Sync ");
                    printf("\n-------------------");
                    bc635pci.cID = COM_FORCEJAMSYNC;
                    SendCommand(1);
#ifdef TEST
                           printf("\n Force Jam Sync.   :  Done");
#endif
                    }break; 
case CCDISP       : {
                    int ctrlcdspl;
                    printf("\n Control   Clock Disciplining ");
                    printf("\n------------------------------");
                    printf("\n Disciplining  Disable  ...(1)");
                    printf("\n Disciplining  Enable   ...(2)");
                    printf("\n Exit .....................(9)");
                    printf("\n Select Your Option .......(");
                    scanf("%d", &ctrlcdspl);

                    switch (ctrlcdspl)
                    {
                    case 1 :
#ifdef TEST
                           printf("\n Control Clock Disciplining   :  Disabled");
#endif
                            bc635pci.cmdarg.UINT8 = DISCIPLINE_DISABLE; break;
                    case 2 :
#ifdef TEST
                           printf("\n Control Clock Disciplining   :  Enabled");
#endif
                            bc635pci.cmdarg.UINT8 = DISCIPLINE_ENABLE ;break;
                    case 9 : break;
                    default : break;
                    }

                    bc635pci.cID = COM_DISCENABLE;
                    if ((ctrlcdspl > 0) && (ctrlcdspl < 3))SendCommand(1);
                    }break; 
case LOADDAC      : {
                    int dacval;
                    printf("\n Load  D-A Convertor ");
                    printf("\n-------------------");
                    printf("\n Enter  Value in HeX form 0.5V(0x0000) - 4.5V(0xFFFF): ");
                    scanf("%4X", &dacval); 
                    bc635pci.cID = COM_SETDAC;
                    bc635pci.cmdarg.UINT16 = dacval;
                    printf("\n Load DAC : %4x", dacval);
                    if ((dacval >= 0x0000) && (dacval <= 0xFFFF)) SendCommand(2);
                    }break; 
case SETDISPGAIN  :  {
                    int dsplgain;
                    printf("\n Set Disciplining Gain");
                    printf("\n----------------------");
                    printf("\n Enter  Value -32768 to +32767): ");
                    scanf("%05d", &dsplgain); 
                    bc635pci.cID = COM_SETDISCGAIN ;
                    bc635pci.cmdarg.UINT8 = dsplgain;
                    printf("\n Set Disciplining Gain : %5d", dsplgain);
                    if ((dsplgain >= 0x0000) && (dsplgain <= 0xFFFF))SendCommand(2);
                    }break; 
case DUMPCFGREG   : DumpDatumPCIDeviceConfiguration(); break; 
case DUMPCTRLREG  : ControlRegDump(); break; 
case DUMPDPRAM    : DatumTFPDPRAMDump(); break; 
case GETBTIME     : {
	                curtime=time(NULL);
                    dtmtime=(*(bctime+13));
                    printf("\n [%x] -> %s", bctime[13],ctime(&dtmtime));
                    } break; 
case SETSTIME     : {

                    curtime=time(NULL);
                    printf("\n System Time (before): %s ", ctime(&curtime));
                    if(BSDgettimeofday (&tp,&tz)!=0) 
						     perror("Not able to get timeof day");
                    printf("\n tv_sec : %ld, tv_usec:%ld", tp.tv_sec, tp.tv_usec);
                    tp.tv_sec  = (*(bc_time+13));
                    tp.tv_usec = (*(bc_time+12));
    
                    if(settimeofday (&tp,0)!=0) 
						perror("Not able to set timeof day");
                    curtime=time(NULL);
                    printf("\n System Time (After): %s ", ctime(&curtime));
     
                     } break; 
case SHOWTIME     : {
	                curtime=time(NULL);
                    dtmtime=(*(bctime+13));
                    printf("\n System Time : %s  Board Time : %s", ctime(&curtime),
						                                          ctime(&dtmtime));
					}break; 

case CMDEXIT      : return (0); break; 

defualt : break;
}

#ifdef DEBUG 
  printf("\n%d", cmdslct);
#endif
getchar();
return (1);
}
/**************************************************************************/
void usage()
{
   printf("\n\n\n Usage : bctime xio  pcislot");
   printf("\n\n\n");
   printf("\n     xio : XIO Node ID on which bc635PCI Card  Installed");
   printf("\n pcislot : PCI Slot Id on XIO Node ");
   printf("\n\n ------------------------------------------------------");
   printf("\n Warning : Please Contact Your System Administrator for ");
   printf("\n vaild Node and Slot ID for bc635PCI card Confiugred on ");
   printf("\n your IRIX system. " );
   printf("\n ------------------------------------------------------");
   printf("\n");
   exit(1);

}
/**************************************************************************/
main(argc, argv)
int argc;
char *argv[];
{ 
int i,j,k;
printf("%c[2J", 27);
printf("\n             bc635PCI Utility for IRIX " );
printf("\n             ------------------------- " );

if (argc < 3)
 {
   if ((getenv(XIONODE) == NULL) || (getenv(PCISLOT) == NULL)) usage();
   else 
   {
   xnid = atoi(getenv(XIONODE));
   pcisid=atoi(getenv(PCISLOT));
   if (  (xnid  < 0)  && ( pcisid <0) ) usage(); 
   }
 }
else
 {
   xnid = atoi(argv[1]);
   pcisid=atoi(argv[2]);
   if (  (xnid  < 0)  && ( pcisid <0) ) usage();
 }  

sprintf(ConfigDevName, "\/hw\/node\/xtalk\/%u\/pci\/%u\/usrpci\/config", xnid, pcisid);
sprintf(Base1Devname, "\/hw\/node\/xtalk\/%u\/pci\/%u\/usrpci\/default", xnid, pcisid);
sprintf(Base2Devname, "\/hw\/node\/xtalk\/%u\/pci\/%u\/usrpci\/base\/1", xnid, pcisid);




#ifdef DEBUG 
printf ("\n\n bc635PCI configured for XIONODE:%d PCISLOT :%d", xnid, pcisid);
printf("\n ConfigDevName %s", ConfigDevName);
printf("\n Base1Devname %s", Base1Devname);
printf("\n Base2Devname %s", Base2Devname);
#endif 



printf("\n Checking  for availability of Datum Device in IRIX Configuration...");
if (!OpenDatumpPCIDevices()) 
   { 
   printf("\n\n\n\n Error : bc635PCI Device  Not  available at XIONODE:%d PCISLOT :%d\n\n\n\n\n", xnid, pcisid);
   exit (1);
   }



printf("\n Checking for Proper BootUp .... ");
if (!ConfigMapDatumPCIDevice ())  exit(1);

devid = (Config[0] << 8) + Config[1];
venid = (Config[2] << 8) + Config[3];


#ifdef DEBUG

printf("\n PCI Card Vendor ID : %04x  Device ID. : %04x", venid, devid);
printf("\n PCI Device ID. :0X%02x%02x", Config[0], Config[1]);
printf("\n PCI Vendor ID. :0X%02x%02x", Config[2], Config[3]);

#endif

if ((devid != BC_VENDOR_ID) || (venid !=   BC635_DEVICE_ID)) 
   {
   printf("\n\n\n\n   Error :  Interface Available at XIONODE:%d \
         PCISLOT :%d is not bc635PCI Device\n\n\n\n\n ", xnid, pcisid);
   closeAndExit();
   exit(1);
   }
if (!MemoryMapDatumPCIDevices ()) exit(1); 

printf("%c[2J", 27);

while (DatumTFPCommadMenu());

if (!closeAndExit()) 
   {
   printf("\n Error in Closing PCI Device Files");
   exit(1);
   }
}
